/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/types.h>
#if defined(CONFIG_CLOCK_CONTROL_NRF2)
#include <hal/nrf_lrcconf.h>
#endif
#include <nrfx_spim.h>
#include <string.h>
#include <stdio.h>
#include <zephyr/drivers/clock_control.h>
#include <zephyr/drivers/clock_control/nrf_clock_control.h>
#if defined(NRF54L15_XXAA)
#include <hal/nrf_clock.h>
#endif /* defined(NRF54L15_XXAA) */
#include <zephyr/drivers/gpio.h>
#include <zephyr/irq.h>
#include <zephyr/logging/log.h>
#include <nrf.h>
#include <esb.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/kernel.h>

#include "encoder.h"
#include "communication.h"
#include "support.h"
#include "common.h"


LOG_MODULE_REGISTER(main, LOG_LEVEL_INF);

#define STACK_SIZE 1024   // Stack-Größe für die Threads
#define PRIORITY 5
#define GPIO_PIN   17 

#define SPIM_SCK_PIN   5//(32 + 5)
#define SPIM_MOSI_PIN  6//(32 + 6)
#define SPIM_MISO_PIN  7//(32 + 7)
#define SPIM_SS_PIN    8//(32 + 8)

#define SPIM_INSTANCE 2

#define GPIO_PIN_NODE DEVICE_DT_GET(DT_NODELABEL(gpio0))

K_SEM_DEFINE(esb_done,0,1);


void communication_event_handler(struct esb_evt const *event)
{
	switch (event->evt_id) {
	case ESB_EVENT_TX_SUCCESS:
		LOG_DBG("TX SUCCESS EVENT");
        set_led(BLUE,true);
        k_sem_give(&esb_done);
		break;
	case ESB_EVENT_TX_FAILED:
		LOG_DBG("TX FAILED EVENT");
        //k_sem_give(&esb_done);
        set_error();
		break;
	case ESB_EVENT_RX_RECEIVED:
		break;
	}
}

encoder_t encoder = {
    .spim_instance = NRFX_SPIM_INSTANCE(SPIM_INSTANCE),
    .gpio_dev = GPIO_PIN_NODE,
};

int main(void)
{
    int err;
    LOG_INF("Initialize");

    // LED Support Initialization
    err = support_init();
    if(err){
        LOG_ERR("Failed support_init");
        return err;
    }

    // Encoder Initialization
    const encoder_cfg_t encoder_cfg = {
        .pin_sclk = SPIM_SCK_PIN,
        .pin_mosi = SPIM_MOSI_PIN,
        .pin_miso = SPIM_MISO_PIN,
        .pin_cs = SPIM_SS_PIN,
        .freq = NRFX_MHZ_TO_HZ(1)
    };
    
    err = encoder_init(&encoder_cfg, &encoder);
    if(err){
        LOG_ERR("Failed encoder_init");
        set_error();
        return err;
    }

    LOG_INF("Enhanced ShockBurst PTX sample");

    err = clocks_start();
    if (err) {
        set_error();
        return err;
    }

    err = communication_init(communication_event_handler);
    if (err) {
        LOG_ERR("ESB initialization failed, err %d", err);
        set_error();
        return err;
    } 

    payload_t payload;
    struct esb_payload esb_message = {
        .length = 6,
        .noack = 0
    };

    while (1) {
        // Reading position 
       payload.err =  encoder_read_pos(&encoder, &payload.postion);

       // Sending position and error
       payload_write(&payload, esb_message.data); 
       payload.err = esb_write_payload(&esb_message);
       if(err < 0){
        set_error();
        set_led(BLUE,false);
        set_led(GREEN,false);
       }
       else{
        set_led(BLUE,true);
        set_led(RED,false);
        set_led(GREEN,false);
       }
       k_sleep(K_MSEC(100));
       esb_flush_tx();
       k_sem_take(&esb_done,K_FOREVER);
    }
}